

Option Explicit

'*** General Module Level Variables ***
Dim ggDB As Database
Dim gDatabaseOpenFlag As Integer      'has the database been opened?
Dim gTablesOpenFlag As Integer        'have the tables been opened?
Dim gErr As Integer                   'for saving the err code
Dim gError As String                  'for saving the error message

'*** Tables ***
Dim tAuthor As Table
Dim tAuthorDs As Dynaset
Dim tAuthorNewDs As Integer

Dim tPublisherComment As Table
Dim tPublisherCommentDs As Dynaset
Dim tPublisherCommentNewDs As Integer

Dim tPublisher As Table
Dim tPublisherDs As Dynaset
Dim tPublisherNewDs As Integer

Dim tTitle As Table
Dim tTitleDs As Dynaset
Dim tTitleNewDs As Integer


Type tpeAuthorKey
   Author_ID As Long
End Type
Type tpeAuthor
   Key As tpeAuthorKey
   Author As String * 255
End Type

Type tpePublisherCommentKey
   PubID As Long
End Type
Type tpePublisherComment
   Key As tpePublisherCommentKey
   Publisher As String * 25
   Comments As String
End Type

Type tpePublisherKey
   PubID As Long
End Type
Type tpePublisher
   Key As tpePublisherKey
   Name As String * 50
   Company_Name As String * 255
   Address As String * 50
   City As String * 20
   State As String * 10
   Zip As String * 15
   Telephone As String * 15
   Fax As String * 15
End Type

Type tpeTitleKey
   ISBN As String * 20
End Type
Type tpeTitle
   Key As tpeTitleKey
   Title As String * 255
   Year_Published As Integer
   Author_ID As Long
   PubID As Long
End Type


Function db_OpenTables() as Integer

'Called to open the tables the first time a call to any API
'function is made.

  On Error GoTo lab_db_OpenTables

  If gTablesOpenFlag Then
     db_OpenTables = DBERR_OK
     Exit Function
  End If
  If Not gDatabaseOpenFlag Then
     db_OpenTables = DBERR_DB_NOT_OPEN
     Exit Function
  End If

  Set tAuthor = ggDB.OpenTable("Authors")
  tAuthor.LockEdits = False
  Set tPublisherComment = ggDB.OpenTable("Publisher Comments")
  tPublisherComment.LockEdits = False
  Set tPublisher = ggDB.OpenTable("Publishers")
  tPublisher.LockEdits = False
  Set tTitle = ggDB.OpenTable("Titles")
  tTitle.LockEdits = False
  gTablesOpenFlag = True
  db_OpenTables = DBERR_OK
  Exit Function

lab_db_OpenTables:
  gErr = Err
  gError = Error
  gTablesOpenFlag = False
  db_OpenTables = DBERR_TABLE_CANNOT_OPEN
  Exit Function

End Function

Function db_CloseTables() as Integer

'Called when the database is closed.

  On Error GoTo lab_db_CloseTables

  If Not gTablesOpenFlag Then
     db_CloseTables = DBERR_OK
     Exit Function
  End If
  If Not gDatabaseOpenFlag Then
     db_CloseTables = DBERR_OK
     Exit Function
  End If

  On Error Resume Next
  tAuthor.Close
  tAuthorDs.Close
  tPublisherComment.Close
  tPublisherCommentDs.Close
  tPublisher.Close
  tPublisherDs.Close
  tTitle.Close
  tTitleDs.Close
  gTablesOpenFlag = False
  db_CloseTables = DBERR_OK
  Exit Function

lab_db_CloseTables:
  gErr = Err
  gError = Error
  gTablesOpenFlag = True
  db_CloseTables = DBERR_TABLE_CANNOT_CLOSE
  Exit Function

End Function

Function db_GetThisAuthor(AuthorRec As tpeAuthor) As Integer

'Returns the value of the entire record as identified in the
'key portion of the record passed.

   Dim Bookmark As Variant
   Dim isKeyValue As Integer
	
   On Error GoTo lab_GetThisAuthor
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetThisAuthor = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tAuthor.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tAuthor.Bookmark
   tAuthor.Seek "=", AuthorRec.Key.Author_ID
   If tAuthor.NoMatch Then
      tAuthor.Bookmark = Bookmark
      db_GetThisAuthor = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Return the information ***
  AuthorRec.Key.Author_ID = IIf(IsNull(tAuthor("Au_ID")), "", tAuthor("Au_ID"))
   AuthorRec.Author = IIf(IsNull(tAuthor("Author")), "", tAuthor("Author"))

   '*** Return Ok ***
   db_GetThisAuthor = DBERR_OK
   Exit Function

lab_GetThisAuthor:
   gErr = Err
   gError = Error
   db_GetThisAuthor = DBERR_REC_NOT_FOUND
   Exit Function

End Function

Function db_GetThisPublisherComment(PublisherCommentRec As tpePublisherComment) As Integer

'Returns the value of the entire record as identified in the
'key portion of the record passed.

   Dim Bookmark As Variant
   Dim isKeyValue As Integer
	
   On Error GoTo lab_GetThisPublisherComment
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetThisPublisherComment = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tPublisherComment.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tPublisherComment.Bookmark
   tPublisherComment.Seek "=", PublisherCommentRec.Key.PubID
   If tPublisherComment.NoMatch Then
      tPublisherComment.Bookmark = Bookmark
      db_GetThisPublisherComment = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Return the information ***
  PublisherCommentRec.Key.PubID = IIf(IsNull(tPublisherComment("PubID")), "", tPublisherComment("PubID"))
   PublisherCommentRec.Publisher = IIf(IsNull(tPublisherComment("Publisher")), "", tPublisherComment("Publisher"))
   PublisherCommentRec.Comments = IIf(IsNull(tPublisherComment("Comments")), "", tPublisherComment("Comments"))

   '*** Return Ok ***
   db_GetThisPublisherComment = DBERR_OK
   Exit Function

lab_GetThisPublisherComment:
   gErr = Err
   gError = Error
   db_GetThisPublisherComment = DBERR_REC_NOT_FOUND
   Exit Function

End Function

Function db_GetThisPublisher(PublisherRec As tpePublisher) As Integer

'Returns the value of the entire record as identified in the
'key portion of the record passed.

   Dim Bookmark As Variant
   Dim isKeyValue As Integer
	
   On Error GoTo lab_GetThisPublisher
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetThisPublisher = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tPublisher.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tPublisher.Bookmark
   tPublisher.Seek "=", PublisherRec.Key.PubID
   If tPublisher.NoMatch Then
      tPublisher.Bookmark = Bookmark
      db_GetThisPublisher = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Return the information ***
  PublisherRec.Key.PubID = IIf(IsNull(tPublisher("PubID")), "", tPublisher("PubID"))
   PublisherRec.Name = IIf(IsNull(tPublisher("Name")), "", tPublisher("Name"))
   PublisherRec.Company_Name = IIf(IsNull(tPublisher("Company Name")), "", tPublisher("Company Name"))
   PublisherRec.Address = IIf(IsNull(tPublisher("Address")), "", tPublisher("Address"))
   PublisherRec.City = IIf(IsNull(tPublisher("City")), "", tPublisher("City"))
   PublisherRec.State = IIf(IsNull(tPublisher("State")), "", tPublisher("State"))
   PublisherRec.Zip = IIf(IsNull(tPublisher("Zip")), "", tPublisher("Zip"))
   PublisherRec.Telephone = IIf(IsNull(tPublisher("Telephone")), "", tPublisher("Telephone"))
   PublisherRec.Fax = IIf(IsNull(tPublisher("Fax")), "", tPublisher("Fax"))

   '*** Return Ok ***
   db_GetThisPublisher = DBERR_OK
   Exit Function

lab_GetThisPublisher:
   gErr = Err
   gError = Error
   db_GetThisPublisher = DBERR_REC_NOT_FOUND
   Exit Function

End Function

Function db_GetThisTitle(TitleRec As tpeTitle) As Integer

'Returns the value of the entire record as identified in the
'key portion of the record passed.

   Dim Bookmark As Variant
   Dim isKeyValue As Integer
	
   On Error GoTo lab_GetThisTitle
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetThisTitle = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tTitle.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tTitle.Bookmark
   tTitle.Seek "=", TitleRec.Key.ISBN
   If tTitle.NoMatch Then
      tTitle.Bookmark = Bookmark
      db_GetThisTitle = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Return the information ***
  TitleRec.Key.ISBN = IIf(IsNull(tTitle("ISBN")), "", tTitle("ISBN"))
   TitleRec.Title = IIf(IsNull(tTitle("Title")), "", tTitle("Title"))
   TitleRec.Year_Published = IIf(IsNull(tTitle("Year Published")), "", tTitle("Year Published"))
   TitleRec.Author_ID = IIf(IsNull(tTitle("Au_ID")), "", tTitle("Au_ID"))
   TitleRec.PubID = IIf(IsNull(tTitle("PubID")), "", tTitle("PubID"))

   '*** Return Ok ***
   db_GetThisTitle = DBERR_OK
   Exit Function

lab_GetThisTitle:
   gErr = Err
   gError = Error
   db_GetThisTitle = DBERR_REC_NOT_FOUND
   Exit Function

End Function


Function db_AddThisAuthor(AuthorRec As tpeAuthor) As Integer

'Add a new record.

  Dim Bookmark As Variant
  Dim isKeyValue As Integer

  On Error GoTo lab_AddThisAuthor
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_AddThisAuthor = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Add the Record ***
  tAuthor.AddNew
  tAuthor("Au_ID") =  AuthorRec.Key.Author_ID
  tAuthor("Author") = AuthorRec.Author
  tAuthor.Update

  '*** Return Ok ***
  db_AddThisAuthor = DBERR_OK
  Exit Function

lab_AddThisAuthor:
  gErr = Err
  gError = Error
  Select Case Err
  Case 3022
     db_AddThisAuthor = DBERR_DUPL_KEY
  Case Else
     db_AddThisAuthor = DBERR_CANNOT_ADD
  End Select
  Exit Function

End Function

Function db_AddThisPublisherComment(PublisherCommentRec As tpePublisherComment) As Integer

'Add a new record.

  Dim Bookmark As Variant
  Dim isKeyValue As Integer

  On Error GoTo lab_AddThisPublisherComment
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_AddThisPublisherComment = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Add the Record ***
  tPublisherComment.AddNew
  tPublisherComment("PubID") =  PublisherCommentRec.Key.PubID
  tPublisherComment("Publisher") = PublisherCommentRec.Publisher
  tPublisherComment("Comments") = PublisherCommentRec.Comments
  tPublisherComment.Update

  '*** Return Ok ***
  db_AddThisPublisherComment = DBERR_OK
  Exit Function

lab_AddThisPublisherComment:
  gErr = Err
  gError = Error
  Select Case Err
  Case 3022
     db_AddThisPublisherComment = DBERR_DUPL_KEY
  Case Else
     db_AddThisPublisherComment = DBERR_CANNOT_ADD
  End Select
  Exit Function

End Function

Function db_AddThisPublisher(PublisherRec As tpePublisher) As Integer

'Add a new record.

  Dim Bookmark As Variant
  Dim isKeyValue As Integer

  On Error GoTo lab_AddThisPublisher
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_AddThisPublisher = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Add the Record ***
  tPublisher.AddNew
  tPublisher("PubID") =  PublisherRec.Key.PubID
  tPublisher("Name") = PublisherRec.Name
  tPublisher("Company Name") = PublisherRec.Company_Name
  tPublisher("Address") = PublisherRec.Address
  tPublisher("City") = PublisherRec.City
  tPublisher("State") = PublisherRec.State
  tPublisher("Zip") = PublisherRec.Zip
  tPublisher("Telephone") = PublisherRec.Telephone
  tPublisher("Fax") = PublisherRec.Fax
  tPublisher.Update

  '*** Return Ok ***
  db_AddThisPublisher = DBERR_OK
  Exit Function

lab_AddThisPublisher:
  gErr = Err
  gError = Error
  Select Case Err
  Case 3022
     db_AddThisPublisher = DBERR_DUPL_KEY
  Case Else
     db_AddThisPublisher = DBERR_CANNOT_ADD
  End Select
  Exit Function

End Function

Function db_AddThisTitle(TitleRec As tpeTitle) As Integer

'Add a new record.

  Dim Bookmark As Variant
  Dim isKeyValue As Integer

  On Error GoTo lab_AddThisTitle
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_AddThisTitle = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Add the Record ***
  tTitle.AddNew
  tTitle("ISBN") =  TitleRec.Key.ISBN
  tTitle("Title") = TitleRec.Title
  tTitle("Year Published") = TitleRec.Year_Published
  tTitle("Au_ID") = TitleRec.Author_ID
  tTitle("PubID") = TitleRec.PubID
  tTitle.Update

  '*** Return Ok ***
  db_AddThisTitle = DBERR_OK
  Exit Function

lab_AddThisTitle:
  gErr = Err
  gError = Error
  Select Case Err
  Case 3022
     db_AddThisTitle = DBERR_DUPL_KEY
  Case Else
     db_AddThisTitle = DBERR_CANNOT_ADD
  End Select
  Exit Function

End Function


Function db_DelListAuthor (SQLFilter As String) As Integer
'Deletes a record(s) by means of an SQL query.
'SQLFilter is the WHERE (excl "WHERE") part of the SQL statement.

  Dim SQLQ As String
  On Error GoTo lab_DelListAuthor
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelListAuthor = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If

  '*** Build the SQL query ***
  Select Case Trim$(SQLFilter)
  Case "ALL"
     SQLQ = "DELETE FROM Authors"
  Case ""
     db_DelListAuthor = DBERR_NO_SQL
     Exit Function
  Case Else
     SQLQ = "DELETE FROM Authors WHERE " & Trim$(SQLFilter)
  End Select

  '*** Execute the query ***
  ggDB.Execute SQLQ

  '*** Exit OK ***
  db_DelListAuthor = DBERR_OK
  Exit Function

lab_DelListAuthor:
  gErr = Err
  gError = Error
  db_DelListAuthor = DBERR_CANNOT_DEL_LIST
  Exit Function

End Function

Function db_DelListPublisherComment (SQLFilter As String) As Integer
'Deletes a record(s) by means of an SQL query.
'SQLFilter is the WHERE (excl "WHERE") part of the SQL statement.

  Dim SQLQ As String
  On Error GoTo lab_DelListPublisherComment
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelListPublisherComment = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If

  '*** Build the SQL query ***
  Select Case Trim$(SQLFilter)
  Case "ALL"
     SQLQ = "DELETE FROM Publisher Comments"
  Case ""
     db_DelListPublisherComment = DBERR_NO_SQL
     Exit Function
  Case Else
     SQLQ = "DELETE FROM Publisher Comments WHERE " & Trim$(SQLFilter)
  End Select

  '*** Execute the query ***
  ggDB.Execute SQLQ

  '*** Exit OK ***
  db_DelListPublisherComment = DBERR_OK
  Exit Function

lab_DelListPublisherComment:
  gErr = Err
  gError = Error
  db_DelListPublisherComment = DBERR_CANNOT_DEL_LIST
  Exit Function

End Function

Function db_DelListPublisher (SQLFilter As String) As Integer
'Deletes a record(s) by means of an SQL query.
'SQLFilter is the WHERE (excl "WHERE") part of the SQL statement.

  Dim SQLQ As String
  On Error GoTo lab_DelListPublisher
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelListPublisher = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If

  '*** Build the SQL query ***
  Select Case Trim$(SQLFilter)
  Case "ALL"
     SQLQ = "DELETE FROM Publishers"
  Case ""
     db_DelListPublisher = DBERR_NO_SQL
     Exit Function
  Case Else
     SQLQ = "DELETE FROM Publishers WHERE " & Trim$(SQLFilter)
  End Select

  '*** Execute the query ***
  ggDB.Execute SQLQ

  '*** Exit OK ***
  db_DelListPublisher = DBERR_OK
  Exit Function

lab_DelListPublisher:
  gErr = Err
  gError = Error
  db_DelListPublisher = DBERR_CANNOT_DEL_LIST
  Exit Function

End Function

Function db_DelListTitle (SQLFilter As String) As Integer
'Deletes a record(s) by means of an SQL query.
'SQLFilter is the WHERE (excl "WHERE") part of the SQL statement.

  Dim SQLQ As String
  On Error GoTo lab_DelListTitle
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelListTitle = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If

  '*** Build the SQL query ***
  Select Case Trim$(SQLFilter)
  Case "ALL"
     SQLQ = "DELETE FROM Titles"
  Case ""
     db_DelListTitle = DBERR_NO_SQL
     Exit Function
  Case Else
     SQLQ = "DELETE FROM Titles WHERE " & Trim$(SQLFilter)
  End Select

  '*** Execute the query ***
  ggDB.Execute SQLQ

  '*** Exit OK ***
  db_DelListTitle = DBERR_OK
  Exit Function

lab_DelListTitle:
  gErr = Err
  gError = Error
  db_DelListTitle = DBERR_CANNOT_DEL_LIST
  Exit Function

End Function


Function db_SetListAuthor (SQLFilter As String, NumFound As Long) As Integer

'Create a transparent dynaset of records to access sequentially using the
'db_GetNext...() API function.  SQLFilter is the WHERE part (excl "WHERE")
'of an SQL statement and could optionally include the "ORDER BY ..." part.
'It could also be the entire SQL statement, starting from the word FROM,
'if SQLQ starts with "FROM...".

   Dim SQLQ As String
   On Error GoTo lab_SetListAuthor
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_SetListAuthor = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** Build the SQL query ***
   If UCase(Left$(LTrim$(SQLFilter),4)) = "FROM" Then
      SQLQ = "SELECT Authors.* " & SQLFilter
   ElseIf UCase(Left$(LTrim$(SQLFilter),8)) = "ORDER BY" Then
      SQLQ = "SELECT Authors.* FROM Authors " & SQLFilter
   Else
      Select Case Trim$(SQLFilter)
      Case "ALL"
         SQLQ = "SELECT Authors.* FROM Authors"
      Case ""
         db_SetListAuthor = DBERR_NO_SQL
         Exit Function
      Case Else
         SQLQ = "SELECT Authors.* FROM Authors WHERE " & Trim$(SQLFilter)
      End Select
   End If

   '*** Create the Dynaset ***
   Set tAuthorDs = ggDB.CreateDynaset(SQLQ)
   tAuthorNewDs = True

   '*** Count the records ***
   If tAuthorDs.RecordCount > 0 Then
      tAuthorDs.MoveLast
      NumFound = tAuthorDs.RecordCount
   Else
      NumFound = 0
   End If

   '*** Return OK ***
   db_SetListAuthor = DBERR_OK
   Exit Function

lab_SetListAuthor:
   gErr = Err
   gError = Error
   db_SetListAuthor = DBERR_CANNOT_SET_LIST
   Exit Function

End Function

Function db_SetListPublisherComment (SQLFilter As String, NumFound As Long) As Integer

'Create a transparent dynaset of records to access sequentially using the
'db_GetNext...() API function.  SQLFilter is the WHERE part (excl "WHERE")
'of an SQL statement and could optionally include the "ORDER BY ..." part.
'It could also be the entire SQL statement, starting from the word FROM,
'if SQLQ starts with "FROM...".

   Dim SQLQ As String
   On Error GoTo lab_SetListPublisherComment
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_SetListPublisherComment = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** Build the SQL query ***
   If UCase(Left$(LTrim$(SQLFilter),4)) = "FROM" Then
      SQLQ = "SELECT Publisher Comments.* " & SQLFilter
   ElseIf UCase(Left$(LTrim$(SQLFilter),8)) = "ORDER BY" Then
      SQLQ = "SELECT Publisher Comments.* FROM Publisher Comments " & SQLFilter
   Else
      Select Case Trim$(SQLFilter)
      Case "ALL"
         SQLQ = "SELECT Publisher Comments.* FROM Publisher Comments"
      Case ""
         db_SetListPublisherComment = DBERR_NO_SQL
         Exit Function
      Case Else
         SQLQ = "SELECT Publisher Comments.* FROM Publisher Comments WHERE " & Trim$(SQLFilter)
      End Select
   End If

   '*** Create the Dynaset ***
   Set tPublisherCommentDs = ggDB.CreateDynaset(SQLQ)
   tPublisherCommentNewDs = True

   '*** Count the records ***
   If tPublisherCommentDs.RecordCount > 0 Then
      tPublisherCommentDs.MoveLast
      NumFound = tPublisherCommentDs.RecordCount
   Else
      NumFound = 0
   End If

   '*** Return OK ***
   db_SetListPublisherComment = DBERR_OK
   Exit Function

lab_SetListPublisherComment:
   gErr = Err
   gError = Error
   db_SetListPublisherComment = DBERR_CANNOT_SET_LIST
   Exit Function

End Function

Function db_SetListPublisher (SQLFilter As String, NumFound As Long) As Integer

'Create a transparent dynaset of records to access sequentially using the
'db_GetNext...() API function.  SQLFilter is the WHERE part (excl "WHERE")
'of an SQL statement and could optionally include the "ORDER BY ..." part.
'It could also be the entire SQL statement, starting from the word FROM,
'if SQLQ starts with "FROM...".

   Dim SQLQ As String
   On Error GoTo lab_SetListPublisher
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_SetListPublisher = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** Build the SQL query ***
   If UCase(Left$(LTrim$(SQLFilter),4)) = "FROM" Then
      SQLQ = "SELECT Publishers.* " & SQLFilter
   ElseIf UCase(Left$(LTrim$(SQLFilter),8)) = "ORDER BY" Then
      SQLQ = "SELECT Publishers.* FROM Publishers " & SQLFilter
   Else
      Select Case Trim$(SQLFilter)
      Case "ALL"
         SQLQ = "SELECT Publishers.* FROM Publishers"
      Case ""
         db_SetListPublisher = DBERR_NO_SQL
         Exit Function
      Case Else
         SQLQ = "SELECT Publishers.* FROM Publishers WHERE " & Trim$(SQLFilter)
      End Select
   End If

   '*** Create the Dynaset ***
   Set tPublisherDs = ggDB.CreateDynaset(SQLQ)
   tPublisherNewDs = True

   '*** Count the records ***
   If tPublisherDs.RecordCount > 0 Then
      tPublisherDs.MoveLast
      NumFound = tPublisherDs.RecordCount
   Else
      NumFound = 0
   End If

   '*** Return OK ***
   db_SetListPublisher = DBERR_OK
   Exit Function

lab_SetListPublisher:
   gErr = Err
   gError = Error
   db_SetListPublisher = DBERR_CANNOT_SET_LIST
   Exit Function

End Function

Function db_SetListTitle (SQLFilter As String, NumFound As Long) As Integer

'Create a transparent dynaset of records to access sequentially using the
'db_GetNext...() API function.  SQLFilter is the WHERE part (excl "WHERE")
'of an SQL statement and could optionally include the "ORDER BY ..." part.
'It could also be the entire SQL statement, starting from the word FROM,
'if SQLQ starts with "FROM...".

   Dim SQLQ As String
   On Error GoTo lab_SetListTitle
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_SetListTitle = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** Build the SQL query ***
   If UCase(Left$(LTrim$(SQLFilter),4)) = "FROM" Then
      SQLQ = "SELECT Titles.* " & SQLFilter
   ElseIf UCase(Left$(LTrim$(SQLFilter),8)) = "ORDER BY" Then
      SQLQ = "SELECT Titles.* FROM Titles " & SQLFilter
   Else
      Select Case Trim$(SQLFilter)
      Case "ALL"
         SQLQ = "SELECT Titles.* FROM Titles"
      Case ""
         db_SetListTitle = DBERR_NO_SQL
         Exit Function
      Case Else
         SQLQ = "SELECT Titles.* FROM Titles WHERE " & Trim$(SQLFilter)
      End Select
   End If

   '*** Create the Dynaset ***
   Set tTitleDs = ggDB.CreateDynaset(SQLQ)
   tTitleNewDs = True

   '*** Count the records ***
   If tTitleDs.RecordCount > 0 Then
      tTitleDs.MoveLast
      NumFound = tTitleDs.RecordCount
   Else
      NumFound = 0
   End If

   '*** Return OK ***
   db_SetListTitle = DBERR_OK
   Exit Function

lab_SetListTitle:
   gErr = Err
   gError = Error
   db_SetListTitle = DBERR_CANNOT_SET_LIST
   Exit Function

End Function


Function db_GetNextAuthor (AuthorRec As tpeAuthor) As Integer

'Is used to traverse the transparent dynaset constructed using
'the db_SetList...() API function.

  On Error GoTo lab_GetNextAuthor

   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetNextAuthor = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** If first call, use MoveFirst and reset flag ***
   If tAuthorNewDs Then
      tAuthorNewDs = False
      tAuthorDs.MoveFirst
   Else
      tAuthorDs.MoveNext
   End If

   '*** Make sure there is a rec to get info from ***
   If tAuthorDs.EOF Or tAuthorDs.BOF Then
      db_GetNextAuthor = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Place record values into passed buffer ***
  AuthorRec.Key.Author_ID = IIf(IsNull(tAuthorDs("Au_ID")), "", tAuthorDs("Au_ID"))
   AuthorRec.Author = IIf(IsNull(tAuthorDs("Author")), "", tAuthorDs("Author"))

   '*** Return OK ***
   db_GetNextAuthor = DBERR_OK
   Exit Function

lab_GetNextAuthor:
   gErr = Err
   gError = Error
   db_GetNextAuthor = DBERR_REC_NOT_FOUND
   Exit Function

End Function

Function db_GetNextPublisherComment (PublisherCommentRec As tpePublisherComment) As Integer

'Is used to traverse the transparent dynaset constructed using
'the db_SetList...() API function.

  On Error GoTo lab_GetNextPublisherComment

   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetNextPublisherComment = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** If first call, use MoveFirst and reset flag ***
   If tPublisherCommentNewDs Then
      tPublisherCommentNewDs = False
      tPublisherCommentDs.MoveFirst
   Else
      tPublisherCommentDs.MoveNext
   End If

   '*** Make sure there is a rec to get info from ***
   If tPublisherCommentDs.EOF Or tPublisherCommentDs.BOF Then
      db_GetNextPublisherComment = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Place record values into passed buffer ***
  PublisherCommentRec.Key.PubID = IIf(IsNull(tPublisherCommentDs("PubID")), "", tPublisherCommentDs("PubID"))
   PublisherCommentRec.Publisher = IIf(IsNull(tPublisherCommentDs("Publisher")), "", tPublisherCommentDs("Publisher"))
   PublisherCommentRec.Comments = IIf(IsNull(tPublisherCommentDs("Comments")), "", tPublisherCommentDs("Comments"))

   '*** Return OK ***
   db_GetNextPublisherComment = DBERR_OK
   Exit Function

lab_GetNextPublisherComment:
   gErr = Err
   gError = Error
   db_GetNextPublisherComment = DBERR_REC_NOT_FOUND
   Exit Function

End Function

Function db_GetNextPublisher (PublisherRec As tpePublisher) As Integer

'Is used to traverse the transparent dynaset constructed using
'the db_SetList...() API function.

  On Error GoTo lab_GetNextPublisher

   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetNextPublisher = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** If first call, use MoveFirst and reset flag ***
   If tPublisherNewDs Then
      tPublisherNewDs = False
      tPublisherDs.MoveFirst
   Else
      tPublisherDs.MoveNext
   End If

   '*** Make sure there is a rec to get info from ***
   If tPublisherDs.EOF Or tPublisherDs.BOF Then
      db_GetNextPublisher = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Place record values into passed buffer ***
  PublisherRec.Key.PubID = IIf(IsNull(tPublisherDs("PubID")), "", tPublisherDs("PubID"))
   PublisherRec.Name = IIf(IsNull(tPublisherDs("Name")), "", tPublisherDs("Name"))
   PublisherRec.Company_Name = IIf(IsNull(tPublisherDs("Company Name")), "", tPublisherDs("Company Name"))
   PublisherRec.Address = IIf(IsNull(tPublisherDs("Address")), "", tPublisherDs("Address"))
   PublisherRec.City = IIf(IsNull(tPublisherDs("City")), "", tPublisherDs("City"))
   PublisherRec.State = IIf(IsNull(tPublisherDs("State")), "", tPublisherDs("State"))
   PublisherRec.Zip = IIf(IsNull(tPublisherDs("Zip")), "", tPublisherDs("Zip"))
   PublisherRec.Telephone = IIf(IsNull(tPublisherDs("Telephone")), "", tPublisherDs("Telephone"))
   PublisherRec.Fax = IIf(IsNull(tPublisherDs("Fax")), "", tPublisherDs("Fax"))

   '*** Return OK ***
   db_GetNextPublisher = DBERR_OK
   Exit Function

lab_GetNextPublisher:
   gErr = Err
   gError = Error
   db_GetNextPublisher = DBERR_REC_NOT_FOUND
   Exit Function

End Function

Function db_GetNextTitle (TitleRec As tpeTitle) As Integer

'Is used to traverse the transparent dynaset constructed using
'the db_SetList...() API function.

  On Error GoTo lab_GetNextTitle

   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_GetNextTitle = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If

   '*** If first call, use MoveFirst and reset flag ***
   If tTitleNewDs Then
      tTitleNewDs = False
      tTitleDs.MoveFirst
   Else
      tTitleDs.MoveNext
   End If

   '*** Make sure there is a rec to get info from ***
   If tTitleDs.EOF Or tTitleDs.BOF Then
      db_GetNextTitle = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Place record values into passed buffer ***
  TitleRec.Key.ISBN = IIf(IsNull(tTitleDs("ISBN")), "", tTitleDs("ISBN"))
   TitleRec.Title = IIf(IsNull(tTitleDs("Title")), "", tTitleDs("Title"))
   TitleRec.Year_Published = IIf(IsNull(tTitleDs("Year Published")), "", tTitleDs("Year Published"))
   TitleRec.Author_ID = IIf(IsNull(tTitleDs("Au_ID")), "", tTitleDs("Au_ID"))
   TitleRec.PubID = IIf(IsNull(tTitleDs("PubID")), "", tTitleDs("PubID"))

   '*** Return OK ***
   db_GetNextTitle = DBERR_OK
   Exit Function

lab_GetNextTitle:
   gErr = Err
   gError = Error
   db_GetNextTitle = DBERR_REC_NOT_FOUND
   Exit Function

End Function


Function db_DelThisAuthor (AuthorKey As tpeAuthorKey) As Integer

'Deletes a record as identified by the key value passed.

  Dim Bookmark As Variant
  Dim ErrCode As Integer

  On Error GoTo lab_DelThisAuthor
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelThisAuthor = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Set the Index ***

  tAuthor.Index = "PrimaryKey"

  '*** Seek the wanted record ***
  Bookmark = tAuthor.Bookmark
  tAuthor.Seek "=", AuthorKey.Author_ID
  If tAuthor.NoMatch Then
     tAuthor.Bookmark = Bookmark
     db_DelThisAuthor = DBERR_REC_NOT_FOUND
     Exit Function
  End If

  '*** Delete the record ***
  tAuthor.Delete

  '*** Exit OK ***
  db_DelThisAuthor = DBERR_OK
  Exit Function

lab_DelThisAuthor:
  gErr = Err
  gError = Error
  db_DelThisAuthor = DBERR_CANNOT_DEL
  Exit Function

End Function

Function db_DelThisPublisherComment (PublisherCommentKey As tpePublisherCommentKey) As Integer

'Deletes a record as identified by the key value passed.

  Dim Bookmark As Variant
  Dim ErrCode As Integer

  On Error GoTo lab_DelThisPublisherComment
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelThisPublisherComment = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Set the Index ***

  tPublisherComment.Index = "PrimaryKey"

  '*** Seek the wanted record ***
  Bookmark = tPublisherComment.Bookmark
  tPublisherComment.Seek "=", PublisherCommentKey.PubID
  If tPublisherComment.NoMatch Then
     tPublisherComment.Bookmark = Bookmark
     db_DelThisPublisherComment = DBERR_REC_NOT_FOUND
     Exit Function
  End If

  '*** Delete the record ***
  tPublisherComment.Delete

  '*** Exit OK ***
  db_DelThisPublisherComment = DBERR_OK
  Exit Function

lab_DelThisPublisherComment:
  gErr = Err
  gError = Error
  db_DelThisPublisherComment = DBERR_CANNOT_DEL
  Exit Function

End Function

Function db_DelThisPublisher (PublisherKey As tpePublisherKey) As Integer

'Deletes a record as identified by the key value passed.

  Dim Bookmark As Variant
  Dim ErrCode As Integer

  On Error GoTo lab_DelThisPublisher
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelThisPublisher = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Set the Index ***

  tPublisher.Index = "PrimaryKey"

  '*** Seek the wanted record ***
  Bookmark = tPublisher.Bookmark
  tPublisher.Seek "=", PublisherKey.PubID
  If tPublisher.NoMatch Then
     tPublisher.Bookmark = Bookmark
     db_DelThisPublisher = DBERR_REC_NOT_FOUND
     Exit Function
  End If

  '*** Delete the record ***
  tPublisher.Delete

  '*** Exit OK ***
  db_DelThisPublisher = DBERR_OK
  Exit Function

lab_DelThisPublisher:
  gErr = Err
  gError = Error
  db_DelThisPublisher = DBERR_CANNOT_DEL
  Exit Function

End Function

Function db_DelThisTitle (TitleKey As tpeTitleKey) As Integer

'Deletes a record as identified by the key value passed.

  Dim Bookmark As Variant
  Dim ErrCode As Integer

  On Error GoTo lab_DelThisTitle
  If Not gTablesOpenFlag Then
     If db_OpenTables() <> DBERR_OK then
        db_DelThisTitle = DBERR_TABLE_CANNOT_OPEN
        Exit Function
     End If
  End If


  '*** Set the Index ***

  tTitle.Index = "PrimaryKey"

  '*** Seek the wanted record ***
  Bookmark = tTitle.Bookmark
  tTitle.Seek "=", TitleKey.ISBN
  If tTitle.NoMatch Then
     tTitle.Bookmark = Bookmark
     db_DelThisTitle = DBERR_REC_NOT_FOUND
     Exit Function
  End If

  '*** Delete the record ***
  tTitle.Delete

  '*** Exit OK ***
  db_DelThisTitle = DBERR_OK
  Exit Function

lab_DelThisTitle:
  gErr = Err
  gError = Error
  db_DelThisTitle = DBERR_CANNOT_DEL
  Exit Function

End Function


Function db_ModThisAuthor(AuthorRec As tpeAuthor) As Integer

   Dim Bookmark As Variant
   Dim isKeyValue As Integer

   On Error GoTo lab_ModThisAuthor
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_ModThisAuthor = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tAuthor.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tAuthor.Bookmark
   tAuthor.Seek "=", AuthorRec.Key.Author_ID
   If tAuthor.NoMatch Then
      tAuthor.Bookmark = Bookmark
      db_ModThisAuthor = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Update the information ***
   tAuthor.Edit
   tAuthor("Au_ID") = AuthorRec.Key.Author_ID
   tAuthor("Author") = AuthorRec.Author
   tAuthor.Update

   '*** Return Ok ***
   db_ModThisAuthor = DBERR_OK
   Exit Function

lab_ModThisAuthor:
   gErr = Err
   gError = Error
   db_ModThisAuthor = DBERR_CANNOT_UPDATE
   Exit Function

End Function

Function db_ModThisPublisherComment(PublisherCommentRec As tpePublisherComment) As Integer

   Dim Bookmark As Variant
   Dim isKeyValue As Integer

   On Error GoTo lab_ModThisPublisherComment
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_ModThisPublisherComment = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tPublisherComment.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tPublisherComment.Bookmark
   tPublisherComment.Seek "=", PublisherCommentRec.Key.PubID
   If tPublisherComment.NoMatch Then
      tPublisherComment.Bookmark = Bookmark
      db_ModThisPublisherComment = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Update the information ***
   tPublisherComment.Edit
   tPublisherComment("PubID") = PublisherCommentRec.Key.PubID
   tPublisherComment("Publisher") = PublisherCommentRec.Publisher
   tPublisherComment("Comments") = PublisherCommentRec.Comments
   tPublisherComment.Update

   '*** Return Ok ***
   db_ModThisPublisherComment = DBERR_OK
   Exit Function

lab_ModThisPublisherComment:
   gErr = Err
   gError = Error
   db_ModThisPublisherComment = DBERR_CANNOT_UPDATE
   Exit Function

End Function

Function db_ModThisPublisher(PublisherRec As tpePublisher) As Integer

   Dim Bookmark As Variant
   Dim isKeyValue As Integer

   On Error GoTo lab_ModThisPublisher
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_ModThisPublisher = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tPublisher.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tPublisher.Bookmark
   tPublisher.Seek "=", PublisherRec.Key.PubID
   If tPublisher.NoMatch Then
      tPublisher.Bookmark = Bookmark
      db_ModThisPublisher = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Update the information ***
   tPublisher.Edit
   tPublisher("PubID") = PublisherRec.Key.PubID
   tPublisher("Name") = PublisherRec.Name
   tPublisher("Company Name") = PublisherRec.Company_Name
   tPublisher("Address") = PublisherRec.Address
   tPublisher("City") = PublisherRec.City
   tPublisher("State") = PublisherRec.State
   tPublisher("Zip") = PublisherRec.Zip
   tPublisher("Telephone") = PublisherRec.Telephone
   tPublisher("Fax") = PublisherRec.Fax
   tPublisher.Update

   '*** Return Ok ***
   db_ModThisPublisher = DBERR_OK
   Exit Function

lab_ModThisPublisher:
   gErr = Err
   gError = Error
   db_ModThisPublisher = DBERR_CANNOT_UPDATE
   Exit Function

End Function

Function db_ModThisTitle(TitleRec As tpeTitle) As Integer

   Dim Bookmark As Variant
   Dim isKeyValue As Integer

   On Error GoTo lab_ModThisTitle
   If Not gTablesOpenFlag Then
      If db_OpenTables() <> DBERR_OK then
         db_ModThisTitle = DBERR_TABLE_CANNOT_OPEN
         Exit Function
      End If
   End If


   '*** Set the Index ***
   tTitle.Index = "PrimaryKey"

   '*** Seek the wanted record ***
   Bookmark = tTitle.Bookmark
   tTitle.Seek "=", TitleRec.Key.ISBN
   If tTitle.NoMatch Then
      tTitle.Bookmark = Bookmark
      db_ModThisTitle = DBERR_REC_NOT_FOUND
      Exit Function
   End If

   '*** Update the information ***
   tTitle.Edit
   tTitle("ISBN") = TitleRec.Key.ISBN
   tTitle("Title") = TitleRec.Title
   tTitle("Year Published") = TitleRec.Year_Published
   tTitle("Au_ID") = TitleRec.Author_ID
   tTitle("PubID") = TitleRec.PubID
   tTitle.Update

   '*** Return Ok ***
   db_ModThisTitle = DBERR_OK
   Exit Function

lab_ModThisTitle:
   gErr = Err
   gError = Error
   db_ModThisTitle = DBERR_CANNOT_UPDATE
   Exit Function

End Function



Function db_OpenDataBase (FileName as String) As Integer

'Opens the physical database file and tables.
'Must be called prior to using the database API.

    Dim ErrCode As Integer

    On Error GoTo labErr_OpenDatabase

    '*** Open Database Object ***
    Set ggDB = OpenDatabase(FileName, False, False, "")

    '*** Return OK ***
    db_OpenDataBase = DBERR_OK
    gDatabaseOpenFlag = True
    Exit Function

'*******************
labErr_OpenDatabase:

    '*** Return Error ***
    gErr = Err
    gError = Error
    gDatabaseOpenFlag = False
    Select Case Err
    Case 3043
       db_OpenDatabase = DBERR_DB_CANNOT_OPEN
    Case 3051
       db_OpenDatabase = DBERR_DB_CANNOT_LOCK
    Case 3024
       db_OpenDatabase = DBERR_DB_INVALID_NAME
    Case 3000
       db_OpenDatabase = DBERR_DB_CANNOT_OPEN
    Case 3049
       db_OpenDatabase = DBERR_DB_CORRUPT
    Case Else
       db_OpenDatabase = DBERR_DB_CANNOT_OPEN
    End Select
    Exit Function

End Function

Function db_CloseDatabase ()

'Closes the database - Always do this before exiting the
'program.

   Dim ErrCode As Integer

   '*** Quit if Database isn't Open ***
   If Not gDatabaseOpenFlag Then
      db_CloseDatabase = DBERR_OK
      Exit Function
   End If

   '*** Close the tables and dynasets ***
   ErrCode = db_CloseTables()
   If ErrCode <> DBERR_OK Then
      GoTo labErr_db_CloseDatabase
   End If

   '*** Close the database ***
   On Err GoTo labErr_db_CloseDatabase
   ggDB.Close

   '*** Return OK **
   db_CloseDatabase = DBERR_OK
   gDatabaseOpenFlag = False
   Exit Function

'***********************
labErr_db_CloseDatabase:
   gErr = Err
   gError = Error
   gDatabaseOpenFlag = True
   db_CloseDatabase = DBERR_DB_CANNOT_CLOSE
   Exit Function

End Function

